﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Battleship.DrawningObjects;
using Battleship.Entities;

namespace Battleship
{
    public partial class FormShipConfig : Form
    {
        /// <summary>
        /// Переменная-выбранная машина
        /// </summary>
        DrawningShip? _ship = null;
        /// <summary>
        /// Событие
        /// </summary>
        public event Action<DrawningShip>? EventAddShip;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormShipConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;

            // TODO buttonCancel.Click with lambda
            buttonCancel.Click += (s, e) => Close(); 

        }
        /// <summary>
        /// Отрисовать машину
        /// </summary>
        private void DrawShip()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _ship?.SetPosition(5, 5);
            _ship?.DrawTransport(gr);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="ev">Привязанный метод</param>
        public void AddEvent(Action<DrawningShip> ev)
        {
            if (EventAddShip == null)
            {
                EventAddShip = ev;
            }
            else
            {
                EventAddShip += ev;
            }
        }
        /// <summary>
        /// Передаем информацию при нажатии на Label
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label)?.DoDragDrop((sender as Label)?.Name, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации (ее типа на соответствие требуемому)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(DataFormats.Text) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Действия при приеме перетаскиваемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data?.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _ship = new DrawningShip((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
                case "labelModifiedObject":
                    _ship = new DrawningBattleship((int)numericUpDownSpeed.Value,
                   (int)numericUpDownWeight.Value, Color.White, Color.Black, checkBoxTower.Checked,
                   checkBoxRocket.Checked, pictureBoxObject.Width, pictureBoxObject.Height);
                    break;
            }
            DrawShip();
        }
        // TODO Реализовать логику смены цветов: основного и дополнительного (для продвинутого объекта)
        /// <summary>
        /// Добавление машины
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>

        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }
        private void labelColor_dragEnter(object sender, DragEventArgs e)
        {
            if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        private void labelColor_dragDrop(object sender, DragEventArgs e)
        {
            if (_ship == null)
                return;
            switch (((Label)sender).Name)
            {
                case "labelBodyColor":
                    _ship?.EntityShip?.setBodyColor((Color)e.Data.GetData(typeof(Color)));
                    break;
                case "labelAddColor":
                    if (!(_ship is DrawningBattleship))
                        return;
                    (_ship.EntityShip as EntityBattleship)?.setAdditionalColor(color: (Color)e.Data.GetData(typeof(Color)));
                    break;
            }
            DrawShip();
        }

        private void buttonAdd_Click_1(object sender, EventArgs e)
        {
            EventAddShip?.Invoke(_ship);
            Close();
        }
    }
}







